﻿using Aspose.Cells;
using Microsoft.AspNetCore.SignalR;
using NPOI.POIFS.Crypt.Dsig;
using OASystem.API.OAMethodLib;
using OASystem.API.OAMethodLib.Hub.HubClients;
using OASystem.API.OAMethodLib.Hub.Hubs;
using OASystem.API.OAMethodLib.QiYeWeChatAPI;
using OASystem.Domain.Dtos.PersonnelModule;
using OASystem.Domain.Entities.PersonnelModule;
using OASystem.Domain.ViewModels.PersonnelModule;
using OASystem.Domain.ViewModels.QiYeWeChat;
using OASystem.Infrastructure.Repositories.PersonnelModule;
using StackExchange.Redis;
using System.Collections.Generic;
using System.Data;
using System.Diagnostics;
using System.Globalization;
using System.IO;

namespace OASystem.API.Controllers
{
    /// <summary>
    /// 人事模块
    /// </summary>
    [Route("api/[controller]/[action]")]
    public class PersonnelModuleController : ControllerBase
    {
        private Result _result;
        private readonly IMapper _mapper;
        private readonly decimal _chengDuMinimumWage = 2100.00M;
        private readonly IQiYeWeChatApiService _qiYeWeChatApiService;
        private readonly WageSheetRepository _wageSheetRep;
        private readonly UsersRepository _usersRep;
        private readonly TaskAllocationRepository _taskAllocationRep;

        private string url;
        private string path;
        private readonly IHubContext<ChatHub, IChatClient> _hubContext;

        /// <summary>
        /// 初始化
        /// </summary>
        /// <param name="qiYeWeChatApiService"></param>
        /// <param name="wageSheetRep"></param>
        /// <param name="usersRep"></param>
        /// <param name="mapper"></param>
        public PersonnelModuleController(IQiYeWeChatApiService qiYeWeChatApiService,WageSheetRepository wageSheetRep, UsersRepository usersRep, IMapper mapper, 
            TaskAllocationRepository taskAllocationRep, IHubContext<ChatHub, IChatClient> hubContext)
        {
            _mapper = mapper;
            _usersRep = usersRep;
            _qiYeWeChatApiService = qiYeWeChatApiService;
            _wageSheetRep = wageSheetRep;
            _result = new Result();

            url = AppSettingsHelper.Get("ExcelBaseUrl");
            path = AppSettingsHelper.Get("ExcelBasePath");
            if (!System.IO.Directory.Exists(path))
            {
                System.IO.Directory.CreateDirectory(path);//不存在就创建文件夹
            }

            this._taskAllocationRep = taskAllocationRep;
            _hubContext = hubContext;
        }

        #region 工资表单

        /// <summary>
        /// 工资 月列表
        /// </summary>
        /// <returns></returns>
        [HttpPost]
        [ProducesResponseType(typeof(JsonView), StatusCodes.Status200OK)]
        public async Task<IActionResult> GetWageSheetMonth()
        {
            string sql = string.Format("Select * From  Pm_WageIssueWorkingDay Where IsDel = 0 Order By YearMonth Desc");
            var data = await _wageSheetRep._sqlSugar.SqlQueryable<WageSheetMonthView>(sql).ToListAsync();

            return Ok(JsonView(true, "查询成功!", data));
        }

        /// <summary>
        /// 工资 工作日信息
        /// 查询
        /// </summary>
        /// <returns></returns>
        [HttpPost]
        [ProducesResponseType(typeof(JsonView), StatusCodes.Status200OK)]
        public async Task<IActionResult> GetWageSheetMonthWorkdays(string startDt, string endDt )
        {

            //参数处理
            string dtFormat = "yyyy-MM-dd";
            DateTime startDt1, endDt1;
            bool startDtIsValid = DateTime.TryParseExact(startDt, dtFormat, CultureInfo.InvariantCulture, DateTimeStyles.None, out startDt1);
            bool endDtIsValid = DateTime.TryParseExact(endDt, dtFormat, CultureInfo.InvariantCulture, DateTimeStyles.None, out endDt1);

            if (!startDtIsValid) return Ok(JsonView(false, "开始日期格式错误!正确时间格式：yyyy-MM-dd  "));
            if (!endDtIsValid) return Ok(JsonView(false, "结束格式错误!正确时间格式：yyyy-MM-dd  "));



            string sql = string.Format(@"Select  * From Sys_Calendar
										 Where Isdel = 0 And Dt between  '{0}' And  '{1}'", startDt, endDt);
            var data = await _wageSheetRep._sqlSugar.SqlQueryable<CalendarInfoView>(sql).ToListAsync();

            return Ok(JsonView(true, "查询成功!", data));

        }

        /// <summary>
        /// 工资 工作日信息
        /// 编辑
        /// </summary>
        /// <returns></returns>
        [HttpPost]
        [ProducesResponseType(typeof(JsonView), StatusCodes.Status200OK)]
        public async Task<IActionResult> GetWageSheetMonthWorkdaysAddOrEdit(WageSheetMonthWorkdaysAddOrEditDto dto)
        {
            //参数处理
            string yearFormat = "yyyy-MM";
            string dtFormat = "yyyy-MM-dd";
            DateTime yearDt, startDt1, endDt1;
            bool yearDtIsValid = DateTime.TryParseExact(dto.YearMonth, yearFormat, CultureInfo.InvariantCulture, DateTimeStyles.None, out yearDt);
            bool startDtIsValid = DateTime.TryParseExact(dto.StartDate, dtFormat, CultureInfo.InvariantCulture, DateTimeStyles.None, out startDt1);
            bool endDtIsValid = DateTime.TryParseExact(dto.EndDate, dtFormat, CultureInfo.InvariantCulture, DateTimeStyles.None, out endDt1);


            if (!yearDtIsValid) return Ok(JsonView(false, "年月日期格式错误!正确时间格式：yyyy-MM  "));
            if (!startDtIsValid) return Ok(JsonView(false, "开始日期格式错误!正确时间格式：yyyy-MM-dd  "));
            if (!endDtIsValid) return Ok(JsonView(false, "结束格式错误!正确时间格式：yyyy-MM-dd  "));

            #region 处理数据
            Pm_WageIssueWorkingDay pm_WageIssueWorkingDay1 = new Pm_WageIssueWorkingDay();
            pm_WageIssueWorkingDay1 = _mapper.Map<Pm_WageIssueWorkingDay>(dto);
            List<Sys_Calendar> sys_Calendars = new List<Sys_Calendar>();
            sys_Calendars = _mapper.Map<List<Sys_Calendar>>(dto.CalendarInfos);
            if (sys_Calendars.Count > 0)
            {
                sys_Calendars = sys_Calendars.OrderBy(it => it.Dt).ToList();
            }

            pm_WageIssueWorkingDay1.Workdays = sys_Calendars.Where(it => it.IsWorkDay == true).ToList().Count();
            foreach (var item in sys_Calendars)
            {
                item.Remark = pm_WageIssueWorkingDay1.Remark;
                item.CreateUserId = pm_WageIssueWorkingDay1.CreateUserId;
                item.CreateTime = pm_WageIssueWorkingDay1.CreateTime;
            }

            #endregion

            var _sqlSugar = _wageSheetRep._sqlSugar;
            _sqlSugar.BeginTran();
            
            try
            {
                List<Sys_Calendar> sys_Calendars_add = new List<Sys_Calendar>();
                List<Sys_Calendar> sys_Calendars_update = new List<Sys_Calendar>();
                sys_Calendars_add = sys_Calendars.Where(it => it.Id == 0).OrderBy(it => it.Dt).ToList();
                sys_Calendars_update = sys_Calendars.Where(it => it.Id != 0).OrderBy(it => it.Dt).ToList();

                int add1 = 0;
                int upd = 0;
                if (sys_Calendars_add.Count > 0)
                {
                    var calendarsAdd = await _sqlSugar.Insertable(sys_Calendars_add).ExecuteReturnIdentityAsync();

                    if (calendarsAdd < 0)
                    {
                        _sqlSugar.RollbackTran();
                        return Ok(JsonView(false, "工作日/节假日/休息日添加失败!"));
                    }
                }
                if (sys_Calendars_update.Count > 0)
                {
                    var calendarsUpdate = await _sqlSugar.Updateable<Sys_Calendar>(sys_Calendars)
                    .UpdateColumns(it => new { it.Dt, it.IsWorkDay, it.IsHoliDay, it.HoliName })
                    .WhereColumns(it => it.Id)
                    .ExecuteCommandAsync();
                    if (calendarsUpdate < 0)
                    {
                        _sqlSugar.RollbackTran();
                        return Ok(JsonView(false, "工作日/节假日/休息日编辑失败!"));
                    }
                }

                var calendarsDatas = await _sqlSugar.Queryable<Sys_Calendar>()
                    .Where(it => Convert.ToDateTime(it.Dt) >= Convert.ToDateTime(dto.StartDate) && Convert.ToDateTime(it.Dt) <= Convert.ToDateTime(dto.EndDate)).ToListAsync();
                if (calendarsDatas.Count < 0)
                {
                    _sqlSugar.RollbackTran();
                    return Ok(JsonView(false, "日期包暂无工作日/节假日/休息日的信息!"));
                }
                //月份表是否存在
                Pm_WageIssueWorkingDay pm_WageIssueWorkingDay = new Pm_WageIssueWorkingDay()
                {
                    YearMonth = dto.YearMonth,
                    StartDate = dto.StartDate,
                    EndDate = dto.EndDate,
                    Workdays = calendarsDatas.Where(it => it.IsWorkDay == true).ToList().Count()
                };

                string sql = string.Format("Select * From Pm_WageIssueWorkingDay Where Isdel = 0 And YearMonth='{0}'", dto.YearMonth);
                var workdsys = await _sqlSugar.SqlQueryable<Pm_WageIssueWorkingDay>(sql).FirstAsync();

                pm_WageIssueWorkingDay.CreateUserId = dto.UserId;
                pm_WageIssueWorkingDay.IsDel = 0;

                if (workdsys == null) //添加
                {
                    int addId = await _sqlSugar.Insertable(pm_WageIssueWorkingDay).ExecuteReturnIdentityAsync();
                    if (addId < 0)
                    {
                        _sqlSugar.RollbackTran();
                        return Ok(JsonView(false, "工作日设置添加失败!"));
                    }
                }
                else //更新
                {
                    int updCount = await _sqlSugar.Updateable(pm_WageIssueWorkingDay)
                        .IgnoreColumns(z => new { z.CreateUserId, z.CreateTime, z.DeleteUserId, z.DeleteTime, z.IsDel })
                        .WhereColumns(it => it.YearMonth)
                        .ExecuteCommandAsync();
                    if (updCount < 0)
                    {
                        _sqlSugar.RollbackTran();
                        return Ok(JsonView(false, "工作日设置编辑失败!"));
                    }
                }

                _sqlSugar.CommitTran();


                return Ok(JsonView(true, "操作成功!"));
            }
            catch (Exception ex)
            {
                _sqlSugar.RollbackTran();
                return Ok(JsonView(false, ex.Message));
            }


        }

        /// <summary>
        /// 工资表单 基础数据源
        /// </summary>
        /// <returns></returns>
        [HttpPost]
        [ProducesResponseType(typeof(JsonView), StatusCodes.Status200OK)]
        public async Task<IActionResult> GetWageSheetBasicsDataSource()
        {
            string companySql = string.Format("Select * From Sys_Company Where IsDel = 0");
            var compnayData = await _wageSheetRep._sqlSugar.SqlQueryable<CompanyNameView>(companySql).ToListAsync();

            string depSql = string.Format("Select * From Sys_Department Where IsDel = 0");
            var depData = await _wageSheetRep._sqlSugar.SqlQueryable<Domain.ViewModels.System.DepartmentView>(depSql).ToListAsync();

            //获取OA系统内所有用户
            var nameData = await _usersRep.GetUserNameList(1);

            if (nameData.Code != 0)
            {
                return Ok(JsonView(false, nameData.Msg));
            }

            var data = new {
                compnayData = compnayData,
                depData = depData,
                userNames = nameData.Data
        };


            return Ok(JsonView(true, "查询成功!", data));
        }

        /// <summary>
        /// 获取工资发放月份
        /// </summary>
        /// <param name="dto"></param>
        /// <returns></returns>
        [HttpPost]
        [ProducesResponseType(typeof(JsonView), StatusCodes.Status200OK)]
        public async Task<IActionResult> GetWageYaerMonths(WageYearDto dto)
        {
            string sql = string.Format(@"Select * From  Pm_WageIssueWorkingDay 
                                         Where Isdel = 0 And YearMonth Like '%{0}%'
                                         Order By YearMonth Asc", dto.Year);

            var data  = await _wageSheetRep._sqlSugar.SqlQueryable<WageYearMonthView>(sql).ToListAsync();
            
           
            return Ok(JsonView(true,"操作成功!", data));
        }

        /// <summary>
        /// 获取工资表单
        /// </summary>
        /// <param name="dto"></param>
        /// <returns></returns>
        [HttpPost]
        [ProducesResponseType(typeof(JsonView), StatusCodes.Status200OK)]
        public async Task<IActionResult> GetWageSheetList(WageSheetListDto dto)
        {
            //参数处理
            string ymFormat = "yyyy-MM";
            DateTime yearMonthDt;
            bool yearMonthDttIsValid = DateTime.TryParseExact(dto.YearMonth, ymFormat, CultureInfo.InvariantCulture, DateTimeStyles.None, out yearMonthDt);
            if (!yearMonthDttIsValid)
            {
                _result.Msg = "年月格式错误!正确时间格式：yyyy-MM  ";
                return Ok(JsonView(false, _result.Msg));
            }

            //获取月工资数据
            string yearMonth = yearMonthDt.ToString("yyyy-MM");

            if (dto.PortType == 1)
            {
                _result = await _wageSheetRep.Get_WageSheet_ListByYearMonthAsync(yearMonth);
                if (_result.Code != 0)
                {
                    return Ok(JsonView(false, _result.Msg));
                }
            }
            else if (dto.PortType == 2)
            { }
            else if (dto.PortType == 3)
            { }
            else
            {
                return Ok(JsonView(false, "请选择正确的端口参数"));
            }

            return Ok(JsonView(true, _result.Msg, _result.Data));
        }

        /// <summary>
        /// 获取工资 详情
        /// </summary>
        /// <param name="dto"></param>
        /// <returns></returns>
        [HttpPost]
        [ProducesResponseType(typeof(JsonView), StatusCodes.Status200OK)]
        public async Task<IActionResult> GetWageSheetById(WageSheetInfoDto dto)
        {
            if (dto.PortType == 1)
            {
                _result = await _wageSheetRep.Get_WageSheet_InfoByIdAsync(dto.Id);
                if (_result.Code != 0)
                {
                    return Ok(JsonView(false, _result.Msg));
                }
            }
            else if (dto.PortType == 2)
            { }
            else if (dto.PortType == 3)
            { }
            else
            {
                return Ok(JsonView(false, "请选择正确的端口参数"));
            }

            return Ok(JsonView(true, _result.Msg, _result.Data));
        }

        /// <summary>
        /// 人事模块 工资表单 删除
        /// </summary>
        /// <param name="dto"></param>
        /// <returns></returns>
        [HttpPost]
        [ProducesResponseType(typeof(JsonView), StatusCodes.Status200OK)]
        public async Task<IActionResult> PostWageSheetDel(WageDelDto dto)
        {
            try
            {
                _result = await _wageSheetRep.Post_WageSheet_DelAsync(dto);
                if (_result.Code != 0)
                {
                    return Ok(JsonView(false, _result.Msg));

                }
            }
            catch (Exception ex)
            {

                return Ok(JsonView(false, ex.Message));
            }

            return Ok(JsonView(true, _result.Msg, _result.Data));
        }

        /// <summary>
        /// 人事模块 工资表单 添加 Or 修改
        /// </summary>
        /// <param name="dto"></param>
        /// <returns></returns>
        [HttpPost]
        [ProducesResponseType(typeof(JsonView), StatusCodes.Status200OK)]
        public async Task<IActionResult> PostWageSheetAddOrEdit(WageAddOrEditDto dto)
        {
            try
            {
                Pm_WageSheet pm_WageSheet = new Pm_WageSheet();
                pm_WageSheet = _mapper.Map<Pm_WageSheet>(dto);
                pm_WageSheet.LastUpdateUserId = dto.CreateUserId;

                #region 计算工资

                //月工资
                decimal salary = pm_WageSheet.Basic + pm_WageSheet.Floats + pm_WageSheet.PostAllowance + pm_WageSheet.InformationSecurityFee + pm_WageSheet.OtherSubsidies;
                //扣款合计
                decimal totalDeduction = pm_WageSheet.SickLeave + pm_WageSheet.SomethingFalse + pm_WageSheet.LateTo + pm_WageSheet.LeaveEarly + pm_WageSheet.Absenteeism + pm_WageSheet.NotPunch +
                                         pm_WageSheet.ReservedFunds + pm_WageSheet.WithholdingInsurance + pm_WageSheet.OtherDeductions + pm_WageSheet.OtherDeductions;

                //实发合计 不含个税
                if (pm_WageSheet.RegularDays >= pm_WageSheet.WorkDays )
                {
                    pm_WageSheet.RegularDays = pm_WageSheet.WorkDays;
                    salary = salary + pm_WageSheet.Mealsupplement + pm_WageSheet.OtherHandle;
                }
                else
                {
                    if (dto.UserId == 21) //21==张海麟
                    {
                        salary = salary + pm_WageSheet.Mealsupplement + pm_WageSheet.OtherHandle;
                    }
                    else
                    {
                        salary = PayrollComputation.ConvertToDecimal(salary / pm_WageSheet.WorkDays * pm_WageSheet.RegularDays + pm_WageSheet.Mealsupplement + pm_WageSheet.OtherHandle);
                    }    
                }
                
                decimal actualTotal = salary - totalDeduction;
                pm_WageSheet.Should = salary;
                pm_WageSheet.TotalDeductions = totalDeduction;
                pm_WageSheet.TotalRealHair = actualTotal - pm_WageSheet.WithholdingTax;
                pm_WageSheet.AfterTax = actualTotal - pm_WageSheet.WithholdingTax;

                #endregion


                _result = await _wageSheetRep.Post_WageSheet_AddOrEditAsync(dto, pm_WageSheet);
                if (_result.Code != 0)
                {
                    return Ok(JsonView(false, _result.Msg));
                }
            }
            catch (Exception ex)
            {

                return Ok(JsonView(false, ex.Message));
            }
                
            return Ok(JsonView(true, _result.Msg, _result.Data));
        }

        /// <summary>
        /// 计算工资
        /// </summary>
        /// <returns></returns>
        [HttpPost]
        [ProducesResponseType(typeof(JsonView), StatusCodes.Status200OK)]
        public async Task<IActionResult> SalaryCalculatorAsync(SalaryCalculatorDto dto)
        {
            Result result = new Result();
            Stopwatch sw = new Stopwatch();
            sw.Start();
            //参数处理
            string ymFormat = "yyyy-MM";
            string dtFormat = "yyyy-MM-dd";
            DateTime yearMonthDt,startDt,endDt;
            bool yearMonthDtIsValid = DateTime.TryParseExact(dto.yearMonth, ymFormat, CultureInfo.InvariantCulture, DateTimeStyles.None, out yearMonthDt);
            bool startDtIsValid = DateTime.TryParseExact(dto.startDt, dtFormat, CultureInfo.InvariantCulture, DateTimeStyles.None, out startDt);
            bool endDtIsValid = DateTime.TryParseExact(dto.endDt, dtFormat, CultureInfo.InvariantCulture, DateTimeStyles.None, out endDt);

            if (!yearMonthDtIsValid) return Ok(JsonView(false, "年月格式错误!正确时间格式：yyyy-MM  "));
            if (!startDtIsValid) return Ok(JsonView(false, "开始日期格式错误!正确时间格式：yyyy-MM-dd  "));
            if (!endDtIsValid) return Ok(JsonView(false, "结束格式错误!正确时间格式：yyyy-MM-dd  "));

            string thisYearMonth = dto.yearMonth;
            string preYearMonth = yearMonthDt.AddMonths(-1).ToString("yyyy-MM");

            //计算本月工资起止时间 比如是2月的1号-28号，那就是2月1号的零点到3月1号的零点 
            DateTime thisStartDt = startDt;
            DateTime thisEndDt = endDt; //

            //本月工资是否有数据 有数据则不计算
            result = await _wageSheetRep.Get_WageSheet_ListByYearMonthAsync(thisYearMonth);
            if (result.Code == 0 )
            {
                return Ok(JsonView(false, thisYearMonth + " 工资数据已存在,若无人员工资请手动添加!"));
            }

            //获取上个月工资信息
            List<Pm_WageSheet> preWageSheetItems = await _wageSheetRep._sqlSugar.Queryable<Pm_WageSheet>().Where(it => it.IsDel == 0 && it.YearMonth == preYearMonth).ToListAsync();
            preWageSheetItems = preWageSheetItems.OrderBy(it => it.UserId).ToList();
            if (preWageSheetItems.Count <= 0) 
            {
                return Ok(JsonView(false, thisYearMonth + " 上月工资数据不存在,请手动添加!"));
            }
            //处理上个月同月同人 多条数据
            List<Pm_WageSheet> preWageSheetItems1 = new List<Pm_WageSheet>();
            preWageSheetItems1 = preWageSheetItems.GroupBy(it => new { it.YearMonth,it.UserId })
                                                  .Select(it => it.FirstOrDefault(item => item.Basic !=0))
                                                  .ToList();

             //获取OA系统内所有用户
            List<UserNameView> userNames =  _usersRep._sqlSugar.SqlQueryable<UserNameView>("Select Id,CnName From Sys_Users").ToList();

            List<Pm_WageSheet> wageSheets = new List<Pm_WageSheet>();

            _result = await PayrollComputation.SalaryCalculatorAsync(preWageSheetItems1, userNames,dto.UserId, thisYearMonth, thisStartDt,thisEndDt);

            #region 批量添加

            if (_result.Code != 0)
            {
                return Ok(JsonView(false, _result.Msg));
            }

            wageSheets = _result.Data;

            var add = await _wageSheetRep._sqlSugar.Insertable(wageSheets).ExecuteCommandAsync();
            if (add <= 0)
            {
                return Ok(JsonView(false, "操作失败!"));
            }

            #endregion

            #region 处理返回数据
            //List <WageSheetItemInfoView> wageSheetItems = new List<WageSheetItemInfoView>();
            //wageSheetItems = _mapper.Map<List<WageSheetItemInfoView>>(wageSheets);
            //wageSheetItems = wageSheetItems.Select(it => 
            //        {
            //            UserNameView? uName1 = new UserNameView();
            //            UserNameView? uName2 = new UserNameView();
            //            uName1 = userNames.Where(it1 => it.UserId == it1.Id).FirstOrDefault();
            //            if (uName1 != null)  it.Name = uName1.CnName;

            //            uName2 = userNames.Where(it1 => it.LastUpdateUserId == it1.Id).FirstOrDefault();
            //            if (uName2 != null) it.LastUpdateUserName = uName2.CnName;

            //            return it; }
            //        ).ToList();
            #endregion

            sw.Stop();
            return Ok(JsonView(true, "操作成功! 耗时：" + (sw.ElapsedMilliseconds/1000) + "s"));
        }

        /// <summary>
        /// 计算工资 By YearMonth And UserId
        /// </summary>
        /// <returns></returns>
        [HttpPost]
        [ProducesResponseType(typeof(JsonView), StatusCodes.Status200OK)]
        public async Task<IActionResult> SalaryCalculatorSingleAsync(SalaryCalculatorSingleDto dto)
        {
            Result result = new Result();
            Stopwatch sw = new Stopwatch();
            sw.Start();

            //参数处理
            string ymFormat = "yyyy-MM";
            string ymdFormat = "yyyy-MM-dd";
            DateTime yearMonthDt,startDt,endDt;
            bool yearMonthDtIsValid = DateTime.TryParseExact(dto.YearMonth, ymFormat, CultureInfo.InvariantCulture, DateTimeStyles.None, out yearMonthDt);
            bool startDtIsValid = DateTime.TryParseExact(dto.StartDate, ymdFormat, CultureInfo.InvariantCulture, DateTimeStyles.None, out startDt);
            bool endDtIsValid = DateTime.TryParseExact(dto.EndDate, ymdFormat, CultureInfo.InvariantCulture, DateTimeStyles.None, out endDt);

            if (!yearMonthDtIsValid) return Ok(JsonView(false, "年月格式错误!正确时间格式：yyyy-MM  "));

            if (!startDtIsValid) return Ok(JsonView(false, "开始时间格式错误!正确时间格式：yyyy-MM-dd  "));

            if (!yearMonthDtIsValid) return Ok(JsonView(false, "结束时间格式错误!正确时间格式：yyyy-MM-dd  "));

            List<Pm_WageSheet> wageSheets = new List<Pm_WageSheet>();
            Pm_WageSheet wageSheet = _mapper.Map<Pm_WageSheet>(dto);
            Pm_WageSheet wageSheet1 = await _wageSheetRep._sqlSugar.Queryable<Pm_WageSheet>().Where(it => it.UserId == dto.UserId && it.YearMonth == dto.YearMonth && it.StartDate == dto.StartDate && it.EndDate == dto.EndDate).FirstAsync();
            if (wageSheet1 != null )
            {
                wageSheet.Id = wageSheet1.Id;
            }

            wageSheets.Add(wageSheet);
            //获取OA系统内所有用户
            List<UserNameView> userNames = _usersRep._sqlSugar.SqlQueryable<UserNameView>("Select Id,CnName From Sys_Users").ToList();

            _result = await PayrollComputation.SalaryCalculatorAsync(wageSheets, userNames, dto.UserId, dto.YearMonth, startDt, endDt);

            if (_result.Code != 0)
            {
                return Ok(JsonView(false, _result.Msg));
            }

            List<Pm_WageSheet> wageSheets1 = new List<Pm_WageSheet>();
            wageSheets1 = _result.Data;

            #region 处理返回数据

            List<WageSheetInfoView> wageSheetItems = new List<WageSheetInfoView>();
            wageSheetItems = _mapper.Map<List<WageSheetInfoView>>(wageSheets1);
            wageSheetItems = wageSheetItems.Select(it =>
                {
                    UserNameView? uName1 = new UserNameView();
                    UserNameView? uName2 = new UserNameView();
                    uName1 = userNames.Where(it1 => it.UserId == it1.Id).FirstOrDefault();
                    if (uName1 != null) it.Name = uName1.CnName;

                    uName2 = userNames.Where(it1 => it.LastUpdateUserId == it1.Id).FirstOrDefault();
                    if (uName2 != null) it.LastUpdateUserName = uName2.CnName;

                    return it;
                }
            ).ToList();
                #endregion

            sw.Stop();
            return Ok(JsonView(true, "操作成功!耗时：" + (sw.ElapsedMilliseconds / 1000) + "s", wageSheetItems[0]));
           
            
        }

        /// <summary>
        /// 导出工资单
        /// </summary>
        /// <returns></returns>
        [HttpPost]
        [ProducesResponseType(typeof(JsonView), StatusCodes.Status200OK)]
        public async Task<IActionResult> ExportWageCard(string yearMonth)
        {
            Result result = new Result();
            Stopwatch sw = new Stopwatch();
            sw.Start();

            //参数处理
            string ymFormat = "yyyy-MM";
            DateTime yearMonthDt;
            bool yearMonthDtIsValid = DateTime.TryParseExact(yearMonth, ymFormat, CultureInfo.InvariantCulture, DateTimeStyles.None, out yearMonthDt);

            if (!yearMonthDtIsValid) return Ok(JsonView(false, "年月格式错误!正确时间格式：yyyy-MM  "));
            //公司部门
            string sql = string.Format(@"Select row_number() over(order by pm_ws.Id) as Row_Number,
                                         	sc.Id as CompanyId,sc.CompanyName,sd.Id as DepId,sd.DepName, 
                                         	sys_u1.CnName Name,sys_u2.CnName LastUpdateUserName,pm_ws.* 
                                         From Pm_WageSheet pm_ws
                                         Left Join Sys_Users sys_u1 On pm_ws.UserId = sys_u1.Id
                                         Left Join Sys_Users sys_u2 On pm_ws.LastUpdateUserId = sys_u2.Id
                                         Left Join Sys_Company sc On  sys_u1.companyId = sc.Id
                                         Left Join Sys_Department sd On sys_u1.DepId = sd.Id
                                         Where pm_ws.IsDel = 0 And pm_ws.YearMonth = '{0}'", yearMonth);
            var wageSheetList = await _wageSheetRep._sqlSugar.SqlQueryable<ExportWageSheetItemView>(sql).ToListAsync();

            if (wageSheetList.Count <= 0)
            {
                return Ok(JsonView(false, yearMonth + "暂无工资数据!"));
            }

            decimal SumPrice = 0.00M;
            foreach (var item in wageSheetList)
            {
                SumPrice += item.AfterTax;
            }

            WorkbookDesigner designer = new WorkbookDesigner();

            designer.Workbook = new Workbook(AppSettingsHelper.Get("ExcelBasePath") + "Template/工资详细清单.xlsx");
            designer.Workbook.Worksheets[0].Name = yearMonth +" 工资单";
            designer.SetDataSource("WageSheet", wageSheetList);
            designer.SetDataSource("YearMonth", yearMonth);
            designer.SetDataSource("StartEndDt", wageSheetList[0].StartDate +" - "+ wageSheetList[0].EndDate);
            designer.SetDataSource("WorkDays", wageSheetList[0].WorkDays);
            designer.SetDataSource("SumPrice", SumPrice);
            designer.SetDataSource("WageSheetTitle", "工资单");//
            designer.Process();

            string fileName = "WageCard/" + yearMonth + "_工资单_" + DateTime.Now.ToString("yyyyMMddHHmmss") + ".xlsx";
            string path = AppSettingsHelper.Get("ExcelBasePath");
            designer.Workbook.Save(path + fileName);
            designer = null;

            string excelPath = AppSettingsHelper.Get("ExcelFtpPath") + fileName;
            string url = AppSettingsHelper.Get("ExcelBaseUrl");
            string fileUrl = url + excelPath;

            sw.Stop();
            return Ok(JsonView(true, "操作成功!耗时：" + (sw.ElapsedMilliseconds / 1000) + "s", new { FileUrl = fileUrl }));
        }

        /// <summary>
        /// 下载个税模板
        /// </summary>
        /// <returns></returns>
        [HttpPost]
        [ProducesResponseType(typeof(JsonView), StatusCodes.Status200OK)]
        public async Task<IActionResult> WageSheetTaxTemplate()
        {
            string serverUrl = AppSettingsHelper.Get("WageSheetExcelBaseUrl");
            var userData = await _usersRep.GetUserNameList(1);

            if (userData.Code == 0)
            {
                var userNames = userData.Data;

                List<string> names = new List<string>();

                List<UserNameView> users = new List<UserNameView>();
                names.Add("管理员");
                names.Add("国交共享号");
                names.Add("人事审核号");
                names.Add("国交主管号");

                List<TaxTemlateViuw> taxs = new List<TaxTemlateViuw>();
                users = JsonConvert.DeserializeObject<List<UserNameView>>(JsonConvert.SerializeObject(userNames));

                foreach (UserNameView item in users)
                {
                    string uName = item.CnName;
                    if (!names.Contains(uName))
                    {
                        taxs.Add(new TaxTemlateViuw { UserName = item.CnName });
                    }
                }

                if (taxs.Count > 0)
                {
                    WorkbookDesigner designer = new WorkbookDesigner();
                    designer.Workbook = new Workbook(AppSettingsHelper.Get("ExcelBasePath") + "Template/个税导入模板.xlsx");
                    designer.Workbook.Worksheets[0].Name = "个税模板";
                    designer.SetDataSource("TaxTemp", taxs);
                    designer.Process();

                    string fileName = "WageSheetTaxFile/个税模板" + DateTime.Now.ToString("yyyyMMddHHmmss") + ".xlsx";
                    string path = AppSettingsHelper.Get("ExcelBasePath");
                    designer.Workbook.Save(path + fileName);
                    designer = null;

                    string excelPath = AppSettingsHelper.Get("ExcelFtpPath") + fileName;
                    string url = AppSettingsHelper.Get("ExcelBaseUrl");
                    string fileUrl = url + excelPath;




                    return Ok(JsonView(true, "操作成功!", new { FileUrl = fileUrl }));
                }

            }


            return Ok(JsonView(false, "操作失败!"));
        }

        /// <summary>
        /// 上传个税
        /// </summary>
        /// <returns></returns>
        [HttpPost]
        [ProducesResponseType(typeof(JsonView), StatusCodes.Status200OK)]
        public async Task<IActionResult> UploadTax(IFormFile file)
        {
            try
            {
                var yearMonth = Request.Headers["YearMonth"].ToString();
                //string yearMonth = "2023-10";
                string ymFormat = "yyyy-MM";
                DateTime yearMonthDt;
                bool yearMonthDtIsValid = DateTime.TryParseExact(yearMonth, ymFormat, CultureInfo.InvariantCulture, DateTimeStyles.None, out yearMonthDt);
                if (!yearMonthDtIsValid) return Ok(JsonView(false, "年月格式错误!正确时间格式：yyyy-MM  "));
                if (file != null)
                {
                    var fileDir = AppSettingsHelper.Get("WageSheetExcelFptPath");
                    //文件名称
                    string projectFileName = file.FileName;

                    //上传的文件的路径
                    string filePath = "";


                    if (!Directory.Exists(fileDir))
                    {
                        Directory.CreateDirectory(fileDir);
                    }

                    //上传的文件的路径
                    filePath = fileDir + $@"\{projectFileName}";

                    if (System.IO.File.Exists(filePath))
                    {
                        //删除文件
                        System.IO.File.Delete(filePath);
                    }

                    using (FileStream fs = System.IO.File.Create(filePath))
                    {
                        file.CopyTo(fs);
                        fs.Flush();
                    }

                    if (System.IO.File.Exists(filePath))
                    {
                        Workbook book = new Workbook(filePath);
                        DataSet dataSet = new DataSet();
                        if (book.Worksheets.Count > 0)
                        {
                            var sheet = book.Worksheets[0];

                            if (sheet != null)
                            {
                                // sheets 中的数据必须存在
                                if (sheet.Cells.MaxDataRow != -1 && sheet.Cells.MaxDataColumn != -1)
                                {
                                    // 方法 ExportDataTable 的参数说明
                                    //  要导出的第一个单元格的行号。
                                    //  要导出的第一个单元格的列号。
                                    //  要导入的行数。
                                    //  要导入的列数。
                                    //  指示第一行的数据是否导出到DataTable的列名。
                                    DataTable dataTable = sheet.Cells.ExportDataTable(0, 0, sheet.Cells.MaxDataRow + 1, sheet.Cells.MaxDataColumn + 1, true);
                                    dataSet.Tables.Add(dataTable);

                                    DataTable taxData = dataSet.Tables[0];

                                    //公司部门
                                    string sql = string.Format(@"Select row_number() over(order by pm_ws.Id) as Row_Number,
                                         	sc.Id as CompanyId,sc.CompanyName,sd.Id as DepId,sd.DepName, 
                                         	sys_u1.CnName Name,sys_u2.CnName LastUpdateUserName,pm_ws.* 
                                         From Pm_WageSheet pm_ws
                                         Left Join Sys_Users sys_u1 On pm_ws.UserId = sys_u1.Id
                                         Left Join Sys_Users sys_u2 On pm_ws.LastUpdateUserId = sys_u2.Id
                                         Left Join Sys_Company sc On  sys_u1.companyId = sc.Id
                                         Left Join Sys_Department sd On sys_u1.DepId = sd.Id
                                         Where pm_ws.IsDel = 0 And pm_ws.YearMonth = '{0}'
                                         Order By UserId Asc ", yearMonth);
                                    var wageSheetList = await _wageSheetRep._sqlSugar.SqlQueryable<WageSheetInfoView>(sql).ToListAsync();

                                    if (wageSheetList.Count <= 0)
                                    {
                                        return Ok(JsonView(false, yearMonth + "工资数据不存在,请先添加工资数据!"));
                                    }

                                    for (int i = 0; i < taxData.Rows.Count; i++)
                                    {
                                        string name = taxData.Rows[i][0].ToString().Trim();

                                        List<WageSheetInfoView> wageSheets = new List<WageSheetInfoView>();
                                        wageSheets = wageSheetList.Where(it => it.Name.Equals(name)).ToList();
                                        if (wageSheets.Count > 0)
                                        {
                                            wageSheetList.Where(it => it.Name.Equals(name))
                                            .Select(it =>
                                             {
                                                 //修改 绩效不等于0.00M的数据
                                                 decimal oldTax = it.WithholdingTax;
                                                 decimal newTax = Convert.ToDecimal(taxData.Rows[i][1].ToString());
                                                 it.WithholdingTax = newTax;
                                                 it.TotalRealHair = it.Should - it.TotalDeductions - newTax;
                                                 return it;
                                             })
                                            .ToList();
                                        }
                                    }
                                    List<Pm_WageSheet> wageSheets1 = new List<Pm_WageSheet>();
                                    wageSheets1 = _mapper.Map<List<Pm_WageSheet>>(wageSheetList);
                                    var updateStatus = _wageSheetRep._sqlSugar
                                        .Updateable(wageSheets1)
                                        .UpdateColumns(it => new { it.WithholdingTax,it.TotalRealHair})
                                        .ExecuteCommand();

                                    if (updateStatus<0)
                                    {
                                        return Ok(JsonView(false, "操作失败！"));
                                    }

                                    if (System.IO.File.Exists(filePath))
                                    {
                                        //删除文件
                                        System.IO.File.Delete(filePath);
                                    }
                                    return Ok(JsonView(true, "操作成功!"));

                                }
                            }
                            else
                            {
                                return Ok(JsonView(false, "工作薄没有数据！"));
                            }


                        }
                    }

                    return Ok(JsonView(true, "上传成功！", projectFileName));
                }
                else
                {
                    return Ok(JsonView(false, "上传失败！"));
                }
            }
            catch (Exception ex)
            {
                return Ok(JsonView(false, "程序错误！"));
                throw;
            }
            return Ok(JsonView(true, "操作成功!"));
        }

        /// <summary>
        /// 打卡记录测试
        /// </summary>
        /// <returns></returns>
        [HttpPost]
        [ProducesResponseType(typeof(JsonView), StatusCodes.Status200OK)]
        public async Task<IActionResult> Test(string startDt,string endDt) 
        {
            UserIdListView userIdListView = await _qiYeWeChatApiService.GetUserIdListAsync();
            if (userIdListView.errcode != 0)
            {
                _result.Msg = "【企业微信】【打卡】【获取员工ID】【Msg】" + userIdListView.errmsg;
                return Ok(JsonView(false, _result.Msg));
            }

            List<string> qyWhchatIdList = new List<string>();
            qyWhchatIdList = userIdListView.dept_user.Select(it => it.userid).ToList();

            var data = await _qiYeWeChatApiService.GetCheckinDataAsync(qyWhchatIdList,2,Convert.ToDateTime(startDt), Convert.ToDateTime(endDt));
            return Ok(JsonView(true, "操作成功!", data.checkindata));
        }

        /// <summary>
        /// 审批详情
        /// </summary>
        /// <returns></returns>
        [HttpPost]
        [ProducesResponseType(typeof(JsonView), StatusCodes.Status200OK)]
        public async Task<IActionResult> PostApprovalDetailAsync(string spNo)
        {
            if (string.IsNullOrEmpty(spNo))
            {
                return Ok(JsonView(false, "审批单号不能为空!!"));
            }
            var data = await _qiYeWeChatApiService.GetApprovalDetailAsync(spNo);
            return Ok(JsonView(true, "操作成功!", data));
        }
        #endregion

        #region 任务单

        /// <summary>
        /// 任务分配 
        /// 基础数据源
        /// </summary>
        /// <returns></returns>
        [HttpPost]
        [ProducesResponseType(typeof(JsonView), StatusCodes.Status200OK)]
        public async Task<IActionResult> PostTaskAllocationInit(TaskAllocationInitDto _dto)
        {
            try
            {
                #region  参数验证
                if (_dto.UserId < 1) return Ok(JsonView(false, "员工Id为空"));
                if (_dto.PageId < 1) _dto.PageId = 172; //任务指派Id

                PageFunAuthViewBase pageFunAuthView = new PageFunAuthViewBase();

                #region 页面操作权限验证
                pageFunAuthView = await GeneralMethod.PostUserPageFuncDatas(_dto.UserId, _dto.PageId);

                if (pageFunAuthView.CheckAuth == 0) return Ok(JsonView(false, "您没有查看权限!"));
                #endregion

                #endregion

                var _view = await _taskAllocationRep._Init(_dto.PortType, _dto.UserId);
                if (_view.Code==0)
                {
                    return Ok(JsonView(true, "查询成功!", _view.Data));
                }

                return Ok(JsonView(false, _view.Msg));
            }
            catch (Exception ex)
            {

                return Ok(JsonView(false, ex.Message));
            }
        }

        /// <summary>
        /// 任务分配 
        /// page
        /// </summary>
        /// <returns></returns>
        [HttpPost]
        [ProducesResponseType(typeof(JsonView), StatusCodes.Status200OK)]
        public async Task<IActionResult> PostTaskAllocationPage(TaskAllocationPageDto _dto)
        {
            try
            {
                #region  参数验证
                if (_dto.UserId < 1) return Ok(JsonView(false, "员工Id为空"));
                if (_dto.PageId < 1) _dto.PageId = 172; //任务指派Id

                PageFunAuthViewBase pageFunAuthView = new PageFunAuthViewBase();

                #region 页面操作权限验证
                pageFunAuthView = await GeneralMethod.PostUserPageFuncDatas(_dto.UserId, _dto.PageId);

                if (pageFunAuthView.CheckAuth == 0) return Ok(JsonView(false, "您没有查看权限!"));
                #endregion

                #endregion

                string whereSql = "";
                #region 分页参数处理


                //类型处理
                if (_dto.Type == 0) whereSql = "";
                else if (_dto.Type == 1) //1 由我指派
                {
                    whereSql = string.Format(@" And ta.CreateUserId = {0} Or (Select COUNT(1) As PeopleNumber From Pm_TaskRelevanceUser Where IsDel = 0 And ta.Id = TAId And UserId = {0}) > 0", _dto.UserId); 
                }
                else if (_dto.Type == 2)// 2 指派给我
                {
                    whereSql = string.Format(@" And (Select COUNT(1) As PeopleNumber From Pm_TaskRelevanceUser Where IsDel = 0 And ta.Id = TAId And UserId = {0}) > 0", _dto.UserId);
                }
                //状态 -1 全部 0 未开始 1 进行中 2 待审核 3 未完成 4 已完成
                if (_dto.Status == -1) //全部
                {
                    whereSql += "";
                }
                else
                {
                    whereSql += string.Format(@" And ta.Status = {0} ", _dto.Status);
                }

                //任务名称
                if (!string.IsNullOrEmpty(_dto.TaskName))
                {
                    whereSql += string.Format(@" And ta.TaskName Like '%{0}%' ", _dto.TaskName);
                }

                #endregion

                string pageSql = string.Format(@"Select * From(
                                                 Select ROW_NUMBER() OVER(ORDER BY ta.CreateTime Desc) AS RowNumber,
                                                 ta.Id,ta.TaskName,ta.TaskPriority,d.DepName,di.TeamName,ta.Status,
                                                 ta.PredictBeginTime,ta.PredictEndTime,u.CnName As CreateUserName,ta.CreateTime,
                                                 (SELECT STUFF(
                                                  (Select  ',' + u.CnName From Pm_TaskRelevanceUser tra
                                                 Left Join Sys_Users u On tra.UserId = u.Id
                                                 Where tra.Isdel = 0 And tra.TAId = ta.Id
                                                 FOR XML PATH('')),1,1,'')) As Participant,
                                                 (SELECT STUFF(
                                                  (Select  ',' + u.CnName From Pm_TaskRelevanceUser tra
                                                 Left Join Sys_Users u On tra.UserId = u.Id
                                                 Where tra.Isdel = 0 And tra.TAId = ta.Id And tra.TaskStatus = 4
                                                 FOR XML PATH('')),1,1,'')) As Consummator
                                                 From Pm_TaskAllocation ta
                                                 Left Join Sys_Department d  On ta.DepId = d.Id 
                                                 Left Join Grp_DelegationInfo di  On ta.DiId = di.Id 
                                                 Left Join Sys_Users u On ta.CreateUserId = u.Id
                                                 Where ta.IsDel = 0 {0} ) As temp ", whereSql);

                RefAsync<int> total = 0;//REF和OUT不支持异步,想要真的异步这是最优解
                var _view = await _taskAllocationRep._sqlSugar.SqlQueryable<TaskListView>(pageSql).ToPageListAsync(_dto.PageIndex, _dto.PageSize, total);//ToPageAsync

                List<int> taskIds = new List<int>();
                taskIds = _view.Select(it => it.Id).ToList();

                string taskerSql = string.Format(@"Select tau.TAId,tau.Id,tau.UserId,u.CnName As UserName,tau.BeginTime,tau.OverTime,tau.TaskStatus,
												  tau.Cause,tau.Score,tau.Remark As ScoreRemark,tau.CreateUserId As TaskCreateUserId
												  From Pm_TaskRelevanceUser tau 
												  Left Join Sys_Users u On tau.UserId = u.Id
												  Where tau.IsDel = 0");

                var taskerData = _taskAllocationRep._sqlSugar.SqlQueryable<TaskerDetailsView>(taskerSql).Where(it => taskIds.Contains(it.TAId)).ToList();
                foreach (var item in _view)
                {
                    item.TaskerDetails = taskerData.Where(it => it.TAId == item.Id).ToList();

                    ////处理任务总状态 And 任务人状态
                    //var taskerStatusData = taskerData.Where(it => it.TAId == item.Id && it.TaskCreateUserId != _dto.UserId && it.UserId == _dto.UserId).FirstOrDefault();
                    //if (taskerStatusData != null)
                    //{
                    //    item.Status = taskerStatusData.TaskStatus;
                    //}
                }


                return Ok(JsonView(true, "查询成功!", _view, total));
            }
            catch (Exception ex)
            {

                return Ok(JsonView(false, ex.Message));
            }
        }

        /// <summary>
        /// 任务分配
        /// 详情
        /// </summary>
        /// <returns></returns>
        [HttpPost]
        [ProducesResponseType(typeof(JsonView), StatusCodes.Status200OK)]
        public async Task<IActionResult> PostTaskAllocationDetails(TaskAllocationDetailsDto _dto)
        {
            try
            {
                #region  参数验证
                if (_dto.UserId < 1) return Ok(JsonView(false, "员工Id为空"));
                if (_dto.PageId < 1) _dto.PageId = 172; //任务指派Id

                PageFunAuthViewBase pageFunAuthView = new PageFunAuthViewBase();

                #region 页面操作权限验证
                pageFunAuthView = await GeneralMethod.PostUserPageFuncDatas(_dto.UserId, _dto.PageId);

                if (pageFunAuthView.CheckAuth == 0) return Ok(JsonView(false, "您没有查看权限!"));
                #endregion

                #endregion

                var _view = await _taskAllocationRep._Details(_dto.PortType, _dto.Id);
                if (_view.Code == 0)
                {
                    return Ok(JsonView(true, "查询成功!", _view.Data));
                }

                return Ok(JsonView(false, _view.Msg));
            }
            catch (Exception ex)
            {
                return Ok(JsonView(false, ex.Message));
            }
        }

        /// <summary>
        /// 任务分配
        /// Add Or Edit
        /// </summary>
        /// <returns></returns>
        [HttpPost]
        [ProducesResponseType(typeof(JsonView), StatusCodes.Status200OK)]
        public async Task<IActionResult> PostTaskAllocationAddOrEdit(TaskAllocationAddOrEditDto _dto)
        {
            try
            {
                #region  参数验证
                if (_dto.UserId < 1) return Ok(JsonView(false, "员工Id为空"));
                if (_dto.PageId < 1) _dto.PageId = 172; //任务指派Id

                PageFunAuthViewBase pageFunAuthView = new PageFunAuthViewBase();

                #region 页面操作权限验证
                pageFunAuthView = await GeneralMethod.PostUserPageFuncDatas(_dto.UserId, _dto.PageId);

                if (_dto.Id == 0)
                {
                    if (pageFunAuthView.AddAuth == 0) return Ok(JsonView(false, "您没有添加权限!"));
                }
                else if (_dto.Id > 0)
                {

                    if (pageFunAuthView.EditAuth == 0) return Ok(JsonView(false, "您没有编辑权限!"));
                }

                
                #endregion

                #endregion

                var _view = await _taskAllocationRep._AddOrEdit(_dto);
                if (_view.Code == 0)
                {

                    if (_dto.Id == 0) //添加提示任务单创建
                    {
                        string title = $"[{_dto.TaskName}] 任务新建成功!";
                        string content = $"[{_dto.TaskName}] 任务新建成功,请前往任务页面查看详情!";

                        await GeneralMethod.MessageIssueAndNotification(MessageTypeEnum.TaskProgressUpdate,title,content,_dto.UserIds);
                    }

                    return Ok(JsonView(true, "操作成功!"));
                }

                return Ok(JsonView(false, _view.Msg));
            }
            catch (Exception ex)
            {
                return Ok(JsonView(false, ex.Message));
            }
        }

        /// <summary>
        /// 任务分配
        /// 状态任务归属人详情
        /// </summary>
        /// <returns></returns>
        [HttpPost]
        [ProducesResponseType(typeof(JsonView), StatusCodes.Status200OK)]
        public async Task<IActionResult> PostTaskAllocationTaskerDetails(TaskerDetailsDto _dto)
        {
            try
            {
                var _view = await _taskAllocationRep._TaskerDetails( _dto.Id);
                if (_view.Code == 0)
                {
                    return Ok(JsonView(true, "操作成功!", _view.Data));
                }

                return Ok(JsonView(false, _view.Msg));
            }
            catch (Exception ex)
            {
                return Ok(JsonView(false, ex.Message));
            }
        }

        /// <summary>
        /// 任务分配
        /// 状态任务归属人设置开始状态
        /// </summary>
        /// <returns></returns>
        [HttpPost]
        [ProducesResponseType(typeof(JsonView), StatusCodes.Status200OK)]
        public async Task<IActionResult> PostTaskAllocationSetStartStatus(TaskerStatusDto _dto)
        {
            try
            {
                var _view = await _taskAllocationRep._TaskerSetStartStatus(_dto.UserId, _dto.Id);
                if (_view.Code == 0)
                {
                    //发送消息
                    var taskData = _taskAllocationRep._sqlSugar.Queryable<Pm_TaskAllocation>().Where(it => it.Id == _dto.Id).First();
                    if (taskData != null)
                    {
                        var taskUserIds = _taskAllocationRep._sqlSugar.Queryable<Pm_TaskRelevanceUser>().Where(it => it.TAId == _dto.Id).Select(it => it.UserId).ToList();
                        if (taskUserIds.Count > 0)
                        {
                            taskUserIds.Remove(_dto.UserId);
                        }

                        var UserName = _taskAllocationRep._sqlSugar.Queryable<Sys_Users>().Where(it => it.Id == _dto.UserId).Select(it => it.CnName).First();

                        string title_createUser = $"[{taskData.TaskName}] 进度更新!";
                        string conten_createUser = $"[{UserName}] 已开始任务,请注意该工作人员任务进度!";

                        await GeneralMethod.MessageIssueAndNotification(MessageTypeEnum.TaskProgressUpdate, title_createUser, conten_createUser, new List<int>() { taskData.CreateUserId }); //创建人发送消息

                        string title = $"[{taskData.TaskName}] 进度更新!";
                        string content = $"[{UserName}] 已开始任务.若需查看,请前往任务页面查看详情!";

                        await GeneralMethod.MessageIssueAndNotification(MessageTypeEnum.TaskProgressUpdate, title, content, taskUserIds); //其他人发送消息


                        string content1 = $"任务已开始,请在规定时间完成！若需查看,请前往任务页面查看详情!";
                        await GeneralMethod.MessageIssueAndNotification(MessageTypeEnum.TaskProgressUpdate, title, content1, new List<int>() { _dto.UserId }); //设置任务人 发送消息

                    }
                   
                    return Ok(JsonView(true, "操作成功!"));
                }

                return Ok(JsonView(false, _view.Msg));
            }
            catch (Exception ex)
            {
                return Ok(JsonView(false, ex.Message));
            }
        }

        /// <summary>
        /// 任务分配
        /// 状态 任务归属人 设置完成状态
        /// </summary>
        /// <returns></returns>
        [HttpPost]
        [ProducesResponseType(typeof(JsonView), StatusCodes.Status200OK)]
        public async Task<IActionResult> PostTaskAllocationSetOverStatus(TaskerStatusDto _dto)
        {
            try
            {
                var _view = await _taskAllocationRep._TaskerSetOverStatus(_dto.UserId, _dto.Id);
                if (_view.Code == 0)
                {
                    //发送消息
                    var taskData = _taskAllocationRep._sqlSugar.Queryable<Pm_TaskAllocation>().Where(it => it.Id == _dto.Id).First();
                    if (taskData != null)
                    {
                        var taskUserIds = _taskAllocationRep._sqlSugar.Queryable<Pm_TaskRelevanceUser>().Where(it => it.TAId == _dto.Id).Select(it => it.UserId).ToList();
                        if (taskUserIds.Count > 0)
                        {
                            taskUserIds.Remove(_dto.UserId);
                        }

                        var UserName = _taskAllocationRep._sqlSugar.Queryable<Sys_Users>().Where(it => it.Id == _dto.UserId).Select(it => it.CnName).First();

                        string title_createUser = $"[{taskData.TaskName}] 进度更新!";
                        string conten_createUser = $"[{UserName}] 已完成任务,请前往任务页面进行审核操作!";

                        await GeneralMethod.MessageIssueAndNotification(MessageTypeEnum.TaskProgressUpdate, title_createUser, conten_createUser, new List<int>() { taskData.CreateUserId }); //创建人发送消息

                        string title = $"[{taskData.TaskName}] 进度更新!";
                        string content = $"[{UserName}] 已完成任务,请注意在规定时间内完成任务.若需查看,请前往任务页面查看详情!";

                        await GeneralMethod.MessageIssueAndNotification(MessageTypeEnum.TaskProgressUpdate, title, content, taskUserIds); //其他人发送消息


                        string content1 = $"任务已完成,等待任务发布人审核！若需查看,请前往任务页面查看详情!";
                        await GeneralMethod.MessageIssueAndNotification(MessageTypeEnum.TaskProgressUpdate, title, content1, new List<int>() { _dto.UserId }); //设置任务人 发送消息

                    }

                    return Ok(JsonView(true, "操作成功!"));
                }

                return Ok(JsonView(false, _view.Msg));
            }
            catch (Exception ex)
            {
                return Ok(JsonView(false, ex.Message));
            }
        }

        /// <summary>
        /// 任务分配
        /// 状态 任务归属人 设置知晓状态
        /// </summary>
        /// <returns></returns>
        [HttpPost]
        [ProducesResponseType(typeof(JsonView), StatusCodes.Status200OK)]
        public async Task<IActionResult> PostTaskAllocationSetHaveStatus(TaskAllocationStatusDto _dto)
        {
            try
            {
                var _view = await _taskAllocationRep._TaskSetHaveStatus(_dto.SubId);
                if (_view.Code == 0)
                {
                    //发送消息
                    var taskUserData = _taskAllocationRep._sqlSugar.Queryable<Pm_TaskRelevanceUser>().Where(it => it.Id == _dto.SubId).First();
                    if (taskUserData != null)
                    {
                        var taskData = _taskAllocationRep._sqlSugar.Queryable<Pm_TaskAllocation>().Where(it => it.Id == taskUserData.TAId).First();
                        
                        if (taskData != null)
                        {

                            var UserName = _taskAllocationRep._sqlSugar.Queryable<Sys_Users>().Where(it => it.Id == taskData.CreateUserId).Select(it => it.CnName).First();

                            string title = $"[{taskData.TaskName}] 进度更新!";
                            string conten_createUser = $"[{UserName}] 已知晓任务.若需查看,请前往任务页面查看详情!";

                            await GeneralMethod.MessageIssueAndNotification(MessageTypeEnum.TaskProgressUpdate, title, conten_createUser, new List<int>() { taskData.CreateUserId }); //创建人发送消息

                            string content = $"请注意任务完成时间！若需查看,请前往任务页面查看详情!";
                            await GeneralMethod.MessageIssueAndNotification(MessageTypeEnum.TaskProgressUpdate, title, content, new List<int>() { taskUserData.UserId }); //设置任务人 发送消息

                        }
                    }

                    return Ok(JsonView(true, "操作成功!"));
                }

                return Ok(JsonView(false, _view.Msg));
            }
            catch (Exception ex)
            {
                return Ok(JsonView(false, ex.Message));
            }
        }

        /// <summary>
        /// 任务分配
        /// 任务发布者 单人设置审批状态
        /// </summary>
        /// <returns></returns>
        [HttpPost]
        [ProducesResponseType(typeof(JsonView), StatusCodes.Status200OK)]
        public async Task<IActionResult> PostTaskAllocationSetAuditStatus(TaskAllocationStatusDto _dto)
        {
            try
            {
                var _view = await _taskAllocationRep._TaskSetAuditStatus(_dto.SubId);
                if (_view.Code == 0)
                {
                    //发送消息
                    var taskUserData = _taskAllocationRep._sqlSugar.Queryable<Pm_TaskRelevanceUser>().Where(it => it.Id == _dto.SubId).First();
                    if (taskUserData != null)
                    {
                        var taskData = _taskAllocationRep._sqlSugar.Queryable<Pm_TaskAllocation>().Where(it => it.Id == taskUserData.TAId).First();

                        if (taskData != null)
                        {
                            string title = $"[{taskData.TaskName}] 进度更新!";
                            string content = $"任务已完成！若需查看,请前往任务页面查看详情!";
                            await GeneralMethod.MessageIssueAndNotification(MessageTypeEnum.TaskProgressUpdate, title, content, new List<int>() { taskUserData.UserId }); //设置任务人 发送消息

                        }
                    }

                    return Ok(JsonView(true, "操作成功!"));
                }

                return Ok(JsonView(false, _view.Msg));
            }
            catch (Exception ex)
            {
                return Ok(JsonView(false, ex.Message));
            }
        }


        /// <summary>
        /// 任务分配
        /// 任务发布者 单人设置未完成状态
        /// </summary>
        /// <returns></returns>
        [HttpPost]
        [ProducesResponseType(typeof(JsonView), StatusCodes.Status200OK)]
        public async Task<IActionResult> PostTaskAllocationSetUnFinishedStatus(TaskAllocationSetUnFinishedStatusDto _dto)
        {
            try
            {
                var _view = await _taskAllocationRep._TaskSetUnFinishedStatus(_dto.SubId,_dto.Cause);
                if (_view.Code == 0)
                {
                    //发送消息
                    var taskUserData = _taskAllocationRep._sqlSugar.Queryable<Pm_TaskRelevanceUser>().Where(it => it.Id == _dto.SubId).First();
                    if (taskUserData != null)
                    {
                        var taskData = _taskAllocationRep._sqlSugar.Queryable<Pm_TaskAllocation>().Where(it => it.Id == taskUserData.TAId).First();

                        if (taskData != null)
                        {
                            string title = $"[{taskData.TaskName}] 进度更新!";
                            string content = $"任务未完成！若需查看,请前往任务页面查看详情!";
                            await GeneralMethod.MessageIssueAndNotification(MessageTypeEnum.TaskProgressUpdate, title, content, new List<int>() { taskUserData.UserId }); //设置任务人 发送消息

                        }
                    }

                    return Ok(JsonView(true, "操作成功!"));
                }

                return Ok(JsonView(false, _view.Msg));
            }
            catch (Exception ex)
            {
                return Ok(JsonView(false, ex.Message));
            }
        }

        ///// <summary>
        ///// 任务分配
        ///// 确认任务是否可操作完成
        ///// </summary>
        ///// <returns></returns>
        //[HttpPost]
        //[ProducesResponseType(typeof(JsonView), StatusCodes.Status200OK)]
        //public async Task<IActionResult> PostTaskAllocationIsConfirmCompletion(TaskAllocationScoreDto _dto)
        //{
        //    try
        //    {
        //        #region  参数验证
        //        if (_dto.UserId < 1) return Ok(JsonView(false, "员工Id为空"));
        //        if (_dto.PageId < 1) _dto.PageId = 172; //任务指派Id

        //        PageFunAuthViewBase pageFunAuthView = new PageFunAuthViewBase();

        //        #region 页面操作权限验证
        //        pageFunAuthView = await GeneralMethod.PostUserPageFuncDatas(_dto.UserId, _dto.PageId);

        //        if (pageFunAuthView.EditAuth == 0) return Ok(JsonView(false, "您没有编辑权限!"));

        //        #endregion

        //        #endregion

        //        var _view = await _taskAllocationRep._TaskConfirmCompletion(_dto.PortType, _dto.Id);
        //        if (_view.Code == 0)
        //        {
        //            return Ok(JsonView(true, "操作成功!"));
        //        }

        //        return Ok(JsonView(false, _view.Msg));
        //    }
        //    catch (Exception ex)
        //    {
        //        return Ok(JsonView(false, ex.Message));
        //    }
        //}

        ///// <summary>
        ///// 任务分配
        ///// 任务确认完成
        ///// </summary>
        ///// <returns></returns>
        //[HttpPost]
        //[ProducesResponseType(typeof(JsonView), StatusCodes.Status200OK)]
        //public async Task<IActionResult> PostTaskAllocationConfirmCompletion(TaskAllocationConfirmCompletionDto _dto)
        //{
        //    try
        //    {
        //        #region  参数验证
        //        if (_dto.UserId < 1) return Ok(JsonView(false, "员工Id为空"));
        //        if (_dto.PageId < 1) _dto.PageId = 172; //任务指派Id

        //        PageFunAuthViewBase pageFunAuthView = new PageFunAuthViewBase();

        //        #region 页面操作权限验证
        //        pageFunAuthView = await GeneralMethod.PostUserPageFuncDatas(_dto.UserId, _dto.PageId);

        //        if (pageFunAuthView.EditAuth == 0) return Ok(JsonView(false, "您没有编辑权限!"));

        //        #endregion

        //        #endregion

        //        var _view = await _taskAllocationRep._TaskConfirmCompletion(_dto.PortType,_dto.Id);
        //        if (_view.Code == 0)
        //        {
        //            return Ok(JsonView(true, "操作成功!"));
        //        }

        //        return Ok(JsonView(false, _view.Msg));
        //    }
        //    catch (Exception ex)
        //    {
        //        return Ok(JsonView(false, ex.Message));
        //    }
        //}

        /// <summary>
        /// 任务分配
        /// 任务发布者 任务评分
        /// </summary>
        /// <returns></returns>
        [HttpPost]
        [ProducesResponseType(typeof(JsonView), StatusCodes.Status200OK)]
        public async Task<IActionResult> PostTaskAllocationScore(TaskAllocationScoreDto _dto)
        {
            try
            {
                #region  参数验证
                if (_dto.UserId < 1) return Ok(JsonView(false, "员工Id为空"));
                if (_dto.PageId < 1) _dto.PageId = 172; //任务指派Id

                PageFunAuthViewBase pageFunAuthView = new PageFunAuthViewBase();

                #region 页面操作权限验证
                pageFunAuthView = await GeneralMethod.PostUserPageFuncDatas(_dto.UserId, _dto.PageId);

                if (pageFunAuthView.EditAuth == 0) return Ok(JsonView(false, "您没有编辑权限!"));

                #endregion

                #endregion

                var _view = await _taskAllocationRep._TaskScore(_dto);
                if (_view.Code == 0)
                {
                    //发送消息
                    var taskUserData = _taskAllocationRep._sqlSugar.Queryable<Pm_TaskRelevanceUser>().Where(it => it.Id == _dto.SubId).First();
                    if (taskUserData != null)
                    {
                        var taskData = _taskAllocationRep._sqlSugar.Queryable<Pm_TaskAllocation>().Where(it => it.Id == taskUserData.TAId).First();

                        if (taskData != null)
                        {
                            string title = $"[{taskData.TaskName}] 进度更新!";
                            string content = $"任务评分已完成！若需查看,请前往任务页面查看详情!";
                            await GeneralMethod.MessageIssueAndNotification(MessageTypeEnum.TaskProgressUpdate, title, content, new List<int>() { taskUserData.UserId }); //设置任务人 发送消息

                        }
                    }

                    return Ok(JsonView(true, "操作成功!"));
                }

                return Ok(JsonView(false, _view.Msg));
            }
            catch (Exception ex)
            {
                return Ok(JsonView(false, ex.Message));
            }
        }

        /// <summary>
        /// 任务分配
        /// 任务发布者 任务终止
        /// </summary>
        /// <returns></returns>
        [HttpPost]
        [ProducesResponseType(typeof(JsonView), StatusCodes.Status200OK)]
        public async Task<IActionResult> PostTaskAllocationTermination(TaskAllocationConfirmCompletionDto _dto)
        {
            try
            {
                #region  参数验证
                if (_dto.UserId < 1) return Ok(JsonView(false, "员工Id为空"));
                if (_dto.PageId < 1) _dto.PageId = 172; //任务指派Id

                PageFunAuthViewBase pageFunAuthView = new PageFunAuthViewBase();

                #region 页面操作权限验证
                pageFunAuthView = await GeneralMethod.PostUserPageFuncDatas(_dto.UserId, _dto.PageId);

                if (pageFunAuthView.EditAuth == 0) return Ok(JsonView(false, "您没有编辑权限!"));

                #endregion

                #endregion

                var _view = await _taskAllocationRep._TaskTermination(_dto.Id);
                if (_view.Code == 0)
                {
                    //发送消息
                    var taskData = _taskAllocationRep._sqlSugar.Queryable<Pm_TaskAllocation>().Where(it => it.Id == _dto.Id).First();
                    if (taskData != null)
                    {
                        var taskUserIds = _taskAllocationRep._sqlSugar.Queryable<Pm_TaskRelevanceUser>().Where(it => it.Id == _dto.Id).Select(it => it.UserId).ToList();
                        taskUserIds.Add(taskData.CreateUserId);


                        string title = $"[{taskData.TaskName}] 进度更新!";
                        string content = $"任务已终止！若需查看,请前往任务页面查看详情!";
                        await GeneralMethod.MessageIssueAndNotification(MessageTypeEnum.TaskProgressUpdate, title, content, taskUserIds); //设置任务人 发送消息

                    }

                    return Ok(JsonView(true, "操作成功!"));
                }

                return Ok(JsonView(false, _view.Msg));
            }
            catch (Exception ex)
            {
                return Ok(JsonView(false, ex.Message));
            }
        }

        /// <summary>
        /// 任务分配
        /// 任务发布者 任务删除
        /// </summary>
        /// <returns></returns>
        [HttpPost]
        [ProducesResponseType(typeof(JsonView), StatusCodes.Status200OK)]
        public async Task<IActionResult> PostTaskAllocationDel(TaskAllocationConfirmCompletionDto _dto)
        {
            try
            {
                #region  参数验证
                if (_dto.UserId < 1) return Ok(JsonView(false, "员工Id为空"));
                if (_dto.PageId < 1) _dto.PageId = 172; //任务指派Id

                PageFunAuthViewBase pageFunAuthView = new PageFunAuthViewBase();

                #region 页面操作权限验证
                pageFunAuthView = await GeneralMethod.PostUserPageFuncDatas(_dto.UserId, _dto.PageId);

                if (pageFunAuthView.DeleteAuth == 0) return Ok(JsonView(false, "您没有删除权限!"));

                #endregion

                #endregion

                var _view = await _taskAllocationRep._TaskDel(_dto.Id);
                if (_view.Code == 0)
                {
                    return Ok(JsonView(true, "操作成功!"));
                }

                return Ok(JsonView(false, _view.Msg));
            }
            catch (Exception ex)
            {
                return Ok(JsonView(false, ex.Message));
            }
        }
        #endregion
    }
}
