﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace OASystem.Domain.Dtos.Groups
{
    /// <summary>
    /// 根据团组id查询费用
    /// </summary>
    public class InvitationOfficialActivitiesListDto
    {
        public int DiId { get; set; }
        public string Inviter { get; set; }
        public string StartInviteTime { get; set; }
        public string EndInviteTime { get; set; }

        public int PageIndex { get; set; } = 1;
        public int PageSize { get; set; } = 10;

        public int IsPaySign { get; set; } = -1;
    }

    /// <summary>
    /// 根据id查询C表数据和商邀费用数据
    /// </summary>
    public class InvitationOfficialActivitiesByIdDto: PortDtoBase
    {
        public int Id { get; set; }
    }
    /// <summary>
    /// 操作参数
    /// </summary>
    public class OpInvitationOfficialActivitiesDto
    {
        /// <summary>
        /// 操作状态
        /// 1 添加 
        /// 2 修改 
        /// </summary>
        public int Status { get; set; }

        //-------------------    C表参数    --------------------------
        //------------------ 消费方式及时间 --------------------------
        /// <summary>
        /// 消费方式
        /// </summary>
        public string ConsumptionPatterns { get; set; }
        /// <summary>
        /// 消费日期
        /// </summary>
        public string ConsumptionDate { get; set; }

        //---------------------payType==刷卡--------------------------
        /// <summary>
        /// 卡类型
        /// </summary>
        public int CTDId { get; set; }
        /// <summary>
        /// 银行卡号
        /// </summary>
        public string BankNo { get; set; }
        /// <summary>
        /// 持卡人姓名
        /// </summary>
        public string CardholderName { get; set; }
        //-------------------payType==转账Or现金 ---------------------
        /// <summary>
        /// 公司银行卡号
        /// </summary>
        public string CompanyBankNo { get; set; }
        /// <summary>
        /// 对方开户行
        /// </summary>
        public string OtherBankName { get; set; }
        /// <summary>
        /// 对方银行账号
        /// </summary>
        public string OtherSideNo { get; set; }
        /// <summary>
        /// 对方姓名
        /// </summary>
        public string OtherSideName { get; set; }
        //---------------------金额-----------------------------------
        /// <summary>
        /// 总计费用
        /// </summary>
        public decimal PayMoney { get; set; }

        /// <summary>
        /// 总计费用币种
        /// </summary>
        public int PaymentCurrency { get; set; }

        /// <summary>
        /// 支付方式
        /// </summary>
        public int PayDId { get; set; }
        /// <summary>
        /// 费用标识
        /// </summary>
        public int OrbitalPrivateTransfer { get; set; }
        /// <summary>
        /// 收款方
        /// </summary>
        public string Payee { get; set; }
        //-------------------商邀参数--------------------------
        /// <summary>
        /// 主键Id
        /// </summary>
        public int Id { get; set; }
        /// <summary>
        /// 团组外键编号
        /// </summary>
        public int DiId { get; set; }

        /// <summary>
        /// 邀请方地区
        /// </summary>
        public string InviterArea { get; set; }

        /// <summary>
        /// 邀请方
        /// </summary>
        public string Inviter { get; set; }

        /// <summary>
        /// 邀请时间
        /// </summary>
        public string InviteTime { get; set; }

        /// <summary>
        /// 附件
        /// </summary>
        public string Attachment { get; set; }

        /// <summary>
        /// 邀请费用
        /// </summary>
        public decimal InviteCost { get; set; }

        /// <summary>
        /// 邀请费用币种
        /// </summary>
        public int InviteCurrency { get; set; }

        /// <summary>
        /// 快递费用
        /// </summary>
        public decimal SendCost { get; set; }

        /// <summary>
        /// 快递费用币种
        /// </summary>
        public int SendCurrency { get; set; }

        /// <summary>
        /// 公务活动费
        /// </summary>
        public decimal EventsCost { get; set; }

        /// <summary>
        /// 公务活动费币种
        /// </summary>
        public int EventsCurrency { get; set; }

        /// <summary>
        /// 公务翻译费
        /// </summary>
        public decimal TranslateCost { get; set; }

        /// <summary>
        /// 公务翻译费币种
        /// </summary>
        [SugarColumn(IsNullable = true, ColumnDataType = "int")]
        public int TranslateCurrency { get; set; }

        /// <summary>
        /// 公务标识（是否到场）
        /// </summary>
        public int IsGoOfficaiaBussiness { get; set; }
        /// <summary>
        /// 创建者Id
        /// </summary>
        public int CreateUserId { get; set; }
        /// <summary>
        /// 备注
        /// </summary>
        public string Remark { get; set; }
        //-----------------------邀请方资料----------
       
        /// 邀请方地址
        public string Address { get; set; }
        /// <summary>
        /// 联系人
        /// </summary>
        public string Contact { get; set; }
        /// <summary>
        /// 职务
        /// </summary>
        public string Job { get; set; }
        /// <summary>
        /// 联系电话
        /// </summary>
        public string Tel { get; set; }
        /// <summary>
        /// 电子邮箱
        /// </summary>
        public string Email { get; set; }
        /// <summary>
        /// 传真号码
        /// </summary>
        public string Fax { get; set; }
        /// <summary>
        /// 其他信息
        /// </summary>
        public string OtherInformation { get; set; }
    }
}
